
#include <Wire.h>
#include <RTC.h>

#define LOW_WATER_LEVEL 490
#define AIR_VALUE 604
#define LED 10
#define BUZZER 9
#define WATER_PUMP 8

static DS3231 RTC;

void printClock(){
    Serial.print(RTC.getDay());
    Serial.print("-");
    Serial.print(RTC.getMonth());
    Serial.print("-");
    Serial.print(RTC.getYear());
    Serial.print(" ");
    Serial.print(RTC.getHours());
    Serial.print(":");
    Serial.print(RTC.getMinutes());
    Serial.print(":");
    Serial.print(RTC.getSeconds());
    Serial.println("");
}

void setupClock(){
  RTC.setHourMode(CLOCK_H24);
  
  RTC.setDay(5);
  RTC.setMonth(1);
  RTC.setYear(2022);

  RTC.setHours(17);
  RTC.setMinutes(04);
  RTC.setSeconds(30);
}

void debug(){
  
  char i;
   
  if(Serial.available() > 0){
    
    i = Serial.read();
    
    if(i == 'a'){
      printClock();
    }

    else if(i == 'b'){
      int moisturePercentage = 0;

      // read soil moisture
      moisturePercentage = analogRead(A0);
      moisturePercentage = map(moisturePercentage, 604, 247, 0, 100);
      Serial.print("Soil moisture: ");
      Serial.println(moisturePercentage);
    }
  }
  
}

void doWatering(){

  int moisturePercentage = 0;
  int valueWater = 0 ;

  // read soil moisture
  moisturePercentage = analogRead(A0);
  moisturePercentage = map(moisturePercentage, 604, 247, 0, 100);
  Serial.print("Soil moisture: ");
  Serial.println(moisturePercentage);

  while (moisturePercentage <= 60){
    
    // read water level
    valueWater = analogRead(A1);
    //Serial.println(valueWater);

    // if low on water supply
    if(valueWater <= LOW_WATER_LEVEL)
      break;

    // turn on the water pump
    digitalWrite(8, LOW);

    // read soil moisture
    moisturePercentage = analogRead(A0);
    moisturePercentage = map(moisturePercentage, 604, 247, 0, 100);
    Serial.print("Soil moisture: ");
    Serial.println(moisturePercentage);
    
  }

  // turn off the water pump
  digitalWrite(8, HIGH);
  
}

void setup(){
  
  Serial.begin(9600);

  // setup clock
  RTC.begin();
  //setupClock();

  // setup motor relay
  pinMode(WATER_PUMP, OUTPUT);
  digitalWrite(WATER_PUMP, HIGH);

  // setup buzzer and led
  pinMode(BUZZER, OUTPUT);
  pinMode(LED, OUTPUT);
  
}

void loop(){
  
  debug();
  // read water level
  int valueWater = analogRead(A1);
  //Serial.println(valueWater);

  // if there is enough water
  if(valueWater > LOW_WATER_LEVEL){
    
    // get current hour
    uint8_t currentHour = RTC.getHours();

    // do if it's watering time
    if(currentHour == 9 || currentHour == 12 || currentHour == 15){

      // get current minute
      uint8_t currentMinute = RTC.getMinutes();

      // threshold for watering time
      if(currentMinute <= 5){
        doWatering();
      }
    }
  }

  // if not enough water
  else{

    while(valueWater <= LOW_WATER_LEVEL){
      
      // turn on buzzer and LED
      digitalWrite(9, HIGH);
      digitalWrite(10, HIGH);  

      // read water level
      valueWater = analogRead(A1);
      Serial.println(valueWater);
      
    }
    
    // turn off buzzer and LED
    digitalWrite(9, LOW);
    digitalWrite(10, LOW);
    
  }
}
